@@ -141,12 +141,18 @@ html_box_text_finalize (GObject *self)
 {
 	HtmlBoxText *text = HTML_BOX_TEXT (self);
 	HtmlBoxTextMaster *master;
+	HtmlBox *box;
 
 	master = text->master;
 	if (master) {
 		html_box_text_free_master (master);
 		g_free (master);
 		text->master = NULL;
+	} else {
+		box = HTML_BOX (self);
+
+		if (box->dom_node)
+			g_object_remove_weak_pointer (G_OBJECT (box->dom_node), (gpointer *) &(box->dom_node));
 	}
 
 	if (text->glyphs) {
@@ -476,7 +482,7 @@ html_box_text_relayout (HtmlBox *self, HtmlRelayout *relayout)
 		    (white_space != master->white_space) ||
 		    relayout->magnification_modified)
 			master->must_relayout = TRUE;
-		
+
 		if (master->must_relayout)
 			html_box_text_recalc_items (text, font_spec, 
 						    white_space, relayout);
@@ -489,7 +495,7 @@ html_box_text_relayout (HtmlBox *self, HtmlRelayout *relayout)
 		text->length = 0;
 		text->item_data = NULL;
 		text->item_offset = 0;
-		
+
 		for (i = master->n_items - 1; i >= 0; i--) {
 			HtmlBoxTextItemData *data = &master->items[i];
 			PangoItem *item = data->item;
@@ -500,6 +506,7 @@ html_box_text_relayout (HtmlBox *self, HtmlRelayout *relayout)
 				child_text = HTML_BOX_TEXT (box_child);
 				if (self->dom_node) {
 					box_child->dom_node = self->dom_node;
+					g_object_add_weak_pointer (G_OBJECT (self->dom_node), (gpointer *) &(box_child->dom_node));
 				}
 				else {
 					html_box_set_style (box_child, HTML_BOX_GET_STYLE (self));
@@ -533,7 +540,7 @@ html_box_text_relayout (HtmlBox *self, HtmlRelayout *relayout)
 
 		while (ptr < end && *ptr != '\n' && *ptr != '\r') 
 			ptr += g_utf8_skip[(unsigned int)*ptr];
-		
+
 		if (ptr != text->canon_text) {
 			guchar *str;
 			gint len;
@@ -662,6 +669,7 @@ html_box_text_relayout (HtmlBox *self, HtmlRelayout *relayout)
 		if (self->dom_node) {
 
 			box_child->dom_node = self->dom_node;
+			g_object_add_weak_pointer (G_OBJECT (self->dom_node), (gpointer *) &(box_child->dom_node));
 		}
 		else {
 			html_box_set_style (box_child, HTML_BOX_GET_STYLE (self));
@@ -685,7 +693,7 @@ html_box_text_relayout (HtmlBox *self, HtmlRelayout *relayout)
 			text->forced_newline = is_newline (*(text->canon_text + text->length - 1)) ? TRUE : FALSE;
 	} else
 		relayout->preserve_leading_space = TRUE;
-		
+
 	html_box_text_get_extents (self, &log_rect);
 	text->ascent = PANGO_PIXELS (PANGO_ASCENT (log_rect));
 	text->descent = PANGO_PIXELS (PANGO_DESCENT (log_rect));
